/************************************************************************/
/*    This program measures the time required by various generators	*/
/*		defined in  RanGen.c  and  RanGen1.c 			*/
/*									*/
/* It also contains the calculation of the Davis-Rabinowitz test	*/
/* integral, the generation of input files for				*/
/* gnuplot to plot "scatter" plots, etc.				*/
/*									*/
/* In the present form some editing is needed to switch between		*/   
/* different functions.							*/
/*									*/
/*			   M. Kolar 1992				*/
/************************************************************************/

#include <errno.h>
#include <time.h>
#include <sys/times.h>
#include "1D.h"

void  err_fread(p) char *p; {}  /* needed for RanGen.c but not used here */
FILE *Listing = stdout, *Output = stdout, *Density = stdout, *Input;

/*-----------------------------------------------------------------------*/
/* Currently available random number generators are defined in RanGen.c. */
/* They require the following declarations:				 */

int rg1Flag, rg2Flag;  /* indicate the two selected generators for
			 particle movement and configuration acceptance	*/
char *rg1 = "\0", *rg2 = SINGLE_STREAM_STRING;
	 /* pointers to verbal descriptions of the two RNG's */

double randNum1[1], randNum2[1];
	/* buffers to store N_fict random numbers generated by the 
	   two generators; these numbers are just enough for one MC cycle */

extern void out_RnGn_state();  /* formated output of the state vectors  */

extern int init_rand_gen();  /* selects and initializes the two generators */

extern double (*RnGn1)(), (*RnGn2)();
	  /* pointers to the two selected RNG */

extern void Eratosthenes();

/*	End of definitions for the file RanGen.c			 */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/* The basic algorithms for the Halton numbers as presented
   in the paper "Fast, portable, and reliable algorithm for ...."
   are in the file RanGen1.c.
   It also contains the generator of the full Halton vector using the 
   first nBase primes, and an unsuccesful "experimental generator"
   based on binary Halton.
*/

extern double init_HaltonBS();
extern double HaltonBS();
extern void init_Halton();
extern double Halton();
extern void init_Halton2();
extern double Halton2();
extern void init_vHalton();
extern void rand_init_vHalton();
extern double vHalton();
extern void init_ExprG();
extern double ExprG();
extern int N0[];
/*	End of definitions for the file RanGen1.c			 */
/*-----------------------------------------------------------------------*/


main(ac, av) int ac; char *av[];
/*============================*/
{
register int i;
int k;

clock_t times(), clock();
double usLast, sysLast, uTo, sTo, cTo, cLast;
struct tms cpuT;

/*#define MAXN	1000000000 */
#define MAXN	1000000
#define GNUM	1	/* Number of generators from RG[] to be tested */

static int state[64];   /* state of random() */
long int ij, kl;

static char *RG[] = { "exprG",  "halton1R2b(1)",
			"halton1R2b()",	  "halton1d(2,3,64)",
		      "halton1R2b(1743)",	"halton1R(2,1743)",
		      "halton1R(3,1)",		"halton1d(3)",
		      "halton1R(5,1)",		"halton1d(5)",
		      "halton1R(7,1)",		"halton1d(7)",
		      "halton1R(11,1)",		"halton1d(11)",
		      "halton1R(13,1)",		"halton1d(13)",
		      "halton1R(17,1)",		"halton1d(17)",
		      "halton1R(19,1)",		"halton1d(19)",
		      "halton1R(23,1)",		"halton1d(23)",
		      "halton1R(107,1)",		"halton1d(107)",
		      "halton1R(659,1)",		"halton1d(659)",
		      "ranmar(1874,8371)",	"random(4123,256)",
		      "erand48(1745,719,3)",	"rand(1)",
		      "halton(1)",		"halton(1)(3)",
		      "ranmar(327,18534)",	"random(1,256)",
		      "erand48(75,813,47345)",	"rand(43123)",
		      "halton(5)",		"halton(10)"		};

PR"\nStarted:  "); PR DateAndTime());
PR"\n\n            Timing of random number generators\n");
PR"            ==================================\n");
PR"               Written by M. Kolar (C) 1992\n\n\n\n");
PR"CPU time needed to generate %d numbers by each generator is measured\n\n\n",
	MAXN);

cLast = clock();
(void)times(&cpuT);
usLast = (double)cpuT.tms_utime / 60.;
sysLast = (double)cpuT.tms_stime / 60.;
PR"Times(): from start: user  %g, system  %g\n\n", usLast, sysLast);


/**{ /* This part can calculate the Davis-Rabinowitz test integral,
/**     see e.g. B. L. Fox, ACM Trans.Math.Soft. 12, 362 (1986)
/**  */
/**  double S = 0, P, pow();
/**  int N = atoi(av[1]), dim = atoi(av[2]);
/**
/** /*   rg1 = av[3];
/**   PE"dim = %d      %s\n", dim, rg1);
/**   init_rand_gen(&rg1, &rg2, 0, 1);
/**   out_RnGn_state(0,stderr);
/** */
/**   PE"dim = %d\n", dim);
/**   if(!strcmp(av[3],"rand")) rand_init_vHalton(dim, atoi(av[4]));
/**   else { if(!strcmp(av[3],"beg")) for(i = 0; i < dim; i++) N0[i] = 1;
/**	  else for(i = 0; i < dim; i++) N0[i] = atoi(av[i + 3]);
/**	  init_vHalton(dim,N0);
/**	}
/**
/** for(i = 1; i <= N; i++)
/**	{ P = 1;
/** /*	  for(k = 0; k < dim; k++) P *= 2 * (*RnGn1)() - 1.;  */
/** /*	  for(k = 0; k < dim; k++) P *= 2 * vHalton() - 1.;
/**	  S += fabs(P);
/** if(i == 500) PE"500: %.12g\n",  S * pow(2.,(double)dim) / 500.);
/** if(i == 1000) PE"1000: %.12g\n",  S * pow(2.,(double)dim) / 1000.);
/** if(i == 7000) PE"7000: %.12g\n",  S * pow(2.,(double)dim) / 7000.);
/** /* if(i == 10000) PE"10000: %g\n",  S * pow(2.,(double)dim) / 10000);*/
/** if(i == 20000) PE"20000: %g\n",  S * pow(2.,(double)dim) / 20000);
/** /* if(i == 30000) PE"30000: %g\n",  S * pow(2.,(double)dim) / 30000);*/
/** if(i == 40000) PE"40000: %g\n",  S * pow(2.,(double)dim) / 40000);
/** /* if(i == 50000) PE"50000: %g\n",  S * pow(2.,(double)dim) / 50000);
/** if(i == 60000) PE"60000: %g\n",  S * pow(2.,(double)dim) / 60000);
/** if(i == 70000) PE"70000: %g\n",  S * pow(2.,(double)dim) / 70000);
/** if(i == 80000) PE"80000: %g\n",  S * pow(2.,(double)dim) / 80000);
/** if(i == 90000) PE"90000: %g\n",  S * pow(2.,(double)dim) / 90000); */
/** if(i == 100000) PE"100000: %g\n",  S * pow(2.,(double)dim) / 100000);
/** /* if(i == 110000) PE"110000: %g\n",  S * pow(2.,(double)dim) / 110000);
/** if(i == 120000) PE"120000: %g\n",  S * pow(2.,(double)dim) / 120000);
/** if(i == 130000) PE"130000: %g\n",  S * pow(2.,(double)dim) / 130000);
/** if(i == 140000) PE"140000: %g\n",  S * pow(2.,(double)dim) / 140000);
/** if(i == 150000) PE"150000: %g\n",  S * pow(2.,(double)dim) / 150000);*/
/** if(i == 200000) PE"200000: %g\n",  S * pow(2.,(double)dim) / 200000);
/** /* if(i == 250000) PE"250000: %g\n",  S * pow(2.,(double)dim) / 250000);
/** if(i == 300000) PE"300000: %g\n",  S * pow(2.,(double)dim) / 300000);
/** if(i == 400000) PE"400000: %g\n",  S * pow(2.,(double)dim) / 400000);*/
/** if(i == 500000) PE"500000: %g\n",  S * pow(2.,(double)dim) / 500000);
/** /* if(i == 600000) PE"600000: %g\n",  S * pow(2.,(double)dim) / 600000);
/** if(i == 700000) PE"700000: %g\n",  S * pow(2.,(double)dim) / 700000);
/** if(i == 800000) PE"800000: %g\n",  S * pow(2.,(double)dim) / 800000);*/
/** if(i == 1000000) { PE"1000000: %g\n",  S * pow(2.,(double)dim) / 1000000);
/**		    break; }
/** if(i == 1200000) PE"1200000: %g\n",  S * pow(2.,(double)dim) / 1200000);
/** if(i == 1500000) PE"1500000: %g\n",  S * pow(2.,(double)dim) / 1500000);
/** if(i == 1750000) PE"1750000: %g\n",  S * pow(2.,(double)dim) / 1750000);
/**	}
/** PE"%d: %g\n",  N, S * pow(2.,(double)dim) / N);
/**
/**exit(0);
/** /* --- end of D-R test integral calculation --- */
/**}
**/

rg1 = "Halton(40, from beginning)";
for(i = 0; i < 40; i++) N0[i] = 1;
init_vHalton(40,N0);

/* init_HaltonBS(atoi(av[1]),1); */
/* init_ExprG(1,0); */
/* srand(atoi(av[1])); */
/*   initstate(atoi(av[1]), state, 256); */
/*state[0] = atoi(av[1]);
state[1] = atoi(av[2]); /*  this is for erand48()  */
/* srand48(atoi(av[1])); */
/* ij = atoi(av[1]);
kl = atoi(av[2]);
rmarin1(&ij, &kl); */

for(k = 0; k < GNUM; k++)
 {
/* rg1 = av[1]; */
   /* rg1 = RG[k]; */    /*   rg2 = RG[1]; */
   PR"==================>  Timing  %s  <==================\n", rg1);
/* if( init_rand_gen(&rg1, &rg2, 0, 1) ) { PR"\n\n"); continue; }
/*   out_RnGn_state(1,stderr); */

   for(i = 0; i < MAXN; i++) (void)(vHalton());

/*   for(i = 0; i < MAXN; i++) (void)(ranmar1());
/*   for(i = 0; i < MAXN; i++) (void)(drand48()); */
/*   for(i = 0; i < MAXN; i++) (void)(erand48(state));
/*   for(i = 0; i < MAXN; i++) (void)(random() / 2147483648.); */
/*   for(i = 0; i < MAXN; i++) (void)(*RnGn1)(); */
/* for(i = 0; i < MAXN; i++) (void)HaltonBS(); */
/* (i = 0; i < atoi(av[2]); i++) PE"%.20g %.20g\n", (*RnGn1)(), (*RnGn1)());
/*for(i = 1; i < MAXN; i++) PE"%.20g %.20g\n", ExprG(), ExprG());
/* for(i = 1; i < 333; i++) (*RnGn1)(); */
/*   for(i = 1; i < 1000; i++) PR" %d  %.20g\n", i, (*RnGn1)()); */
/*   for(i = 0; i < MAXN; i++)
	{ double rn1, rn2;
/*	  if(fabs((rn1 = (*RnGn1)()) - (rn2 = (*RnGn2)())) > 1e-15) */
/*	  if( (rn1 = (*RnGn1)()) != (rn2 = (*RnGn2)()) ) */
/*  	  if( (rn1 = (*RnGn2)()) != (rn2 = Halton2()) )
		  PR"%d  %.20g %.20g %g\n", i, rn1, rn2, rn1 / rn2);
        }
   out_RnGn_state(1,stderr);
*/
   PR"\nReal time:  "); PR DateAndTime());
   cTo = (double)clock() / CLOCKS_PER_SEC;
   PR"\nClock(): total  %g\n", cTo - cLast);
   cLast = cTo;
   (void)times(&cpuT);
   uTo = (double)cpuT.tms_utime / 60.;
   sTo = (double)cpuT.tms_stime / 60.;
   PR"Times(): user  %g, system  %g, total  %g\n",
	uTo - usLast , sTo - sysLast, uTo + sTo - (usLast + sysLast));
   usLast = uTo;
   sysLast = sTo;
   PR"\n\n\n");
   (void)fflush(Listing);
 }

PR"Finished:  "); PR DateAndTime());
PR"\n\nWhole job: Clock(): total  %g\n", cTo);
PR"           Times(): user  %g, system  %g, total  %g\n",
					uTo, sTo, uTo + sTo);
PR"\n");

}
